-- Do not edit.  Generated on Tue Feb 11 11:36:23 2020 by tgingold
-- With Cheby 1.4.dev0 and these options:
--  -i fip_urv_regs.cheby --gen-hdl fip_urv_regs.vhd


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.wishbone_pkg.all;

entity fip_urv_regs is
  port (
    rst_n_i              : in    std_logic;
    clk_i                : in    std_logic;
    wb_i                 : in    t_wishbone_slave_in;
    wb_o                 : out   t_wishbone_slave_out;

    -- PLC processor control
    -- Put the PLC processor on reset
    plc_ctrl_rst_o       : out   std_logic;

    -- Status bits from NanoFIP
    fip_status_var1_rdy_i : in    std_logic;
    fip_status_var3_rdy_i : in    std_logic;

    -- presence lines for boards
    presence_en_i        : in    std_logic_vector(7 downto 0);

    -- led
    leds_val_o           : out   std_logic_vector(5 downto 0);

    -- REG pins_0
    boards_pins_0_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_1
    boards_pins_1_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_2
    boards_pins_2_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_3
    boards_pins_3_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_4
    boards_pins_4_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_5
    boards_pins_5_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_6
    boards_pins_6_i      : in    std_logic_vector(31 downto 0);

    -- REG pins_7
    boards_pins_7_i      : in    std_logic_vector(31 downto 0);

    -- NanoFIP internal memory/registers
    fip_reg_i            : in    t_wishbone_master_in;
    fip_reg_o            : out   t_wishbone_master_out;

    -- Memory of the PLC urv
    plc_mem_i            : in    t_wishbone_master_in;
    plc_mem_o            : out   t_wishbone_master_out
  );
end fip_urv_regs;

architecture syn of fip_urv_regs is
  signal adr_int                        : std_logic_vector(13 downto 2);
  signal rd_req_int                     : std_logic;
  signal wr_req_int                     : std_logic;
  signal rd_ack_int                     : std_logic;
  signal wr_ack_int                     : std_logic;
  signal wb_en                          : std_logic;
  signal ack_int                        : std_logic;
  signal wb_rip                         : std_logic;
  signal wb_wip                         : std_logic;
  signal plc_ctrl_rst_reg               : std_logic;
  signal plc_ctrl_wreq                  : std_logic;
  signal plc_ctrl_wack                  : std_logic;
  signal leds_val_reg                   : std_logic_vector(5 downto 0);
  signal leds_wreq                      : std_logic;
  signal leds_wack                      : std_logic;
  signal fip_reg_re                     : std_logic;
  signal fip_reg_we                     : std_logic;
  signal fip_reg_wt                     : std_logic;
  signal fip_reg_rt                     : std_logic;
  signal fip_reg_tr                     : std_logic;
  signal fip_reg_wack                   : std_logic;
  signal fip_reg_rack                   : std_logic;
  signal plc_mem_re                     : std_logic;
  signal plc_mem_we                     : std_logic;
  signal plc_mem_wt                     : std_logic;
  signal plc_mem_rt                     : std_logic;
  signal plc_mem_tr                     : std_logic;
  signal plc_mem_wack                   : std_logic;
  signal plc_mem_rack                   : std_logic;
  signal rd_ack_d0                      : std_logic;
  signal rd_dat_d0                      : std_logic_vector(31 downto 0);
  signal wr_req_d0                      : std_logic;
  signal wr_adr_d0                      : std_logic_vector(13 downto 2);
  signal wr_dat_d0                      : std_logic_vector(31 downto 0);
  signal wr_sel_d0                      : std_logic_vector(3 downto 0);
begin

  -- WB decode signals
  adr_int <= wb_i.adr(13 downto 2);
  wb_en <= wb_i.cyc and wb_i.stb;

  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wb_rip <= '0';
      else
        wb_rip <= (wb_rip or (wb_en and not wb_i.we)) and not rd_ack_int;
      end if;
    end if;
  end process;
  rd_req_int <= (wb_en and not wb_i.we) and not wb_rip;

  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wb_wip <= '0';
      else
        wb_wip <= (wb_wip or (wb_en and wb_i.we)) and not wr_ack_int;
      end if;
    end if;
  end process;
  wr_req_int <= (wb_en and wb_i.we) and not wb_wip;

  ack_int <= rd_ack_int or wr_ack_int;
  wb_o.ack <= ack_int;
  wb_o.stall <= not ack_int and wb_en;
  wb_o.rty <= '0';
  wb_o.err <= '0';

  -- pipelining for wr-in+rd-out
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        rd_ack_int <= '0';
        wr_req_d0 <= '0';
      else
        rd_ack_int <= rd_ack_d0;
        wb_o.dat <= rd_dat_d0;
        wr_req_d0 <= wr_req_int;
        wr_adr_d0 <= adr_int;
        wr_dat_d0 <= wb_i.dat;
        wr_sel_d0 <= wb_i.sel;
      end if;
    end if;
  end process;

  -- Register plc_ctrl
  plc_ctrl_rst_o <= plc_ctrl_rst_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        plc_ctrl_rst_reg <= '1';
        plc_ctrl_wack <= '0';
      else
        if plc_ctrl_wreq = '1' then
          plc_ctrl_rst_reg <= wr_dat_d0(0);
        end if;
        plc_ctrl_wack <= plc_ctrl_wreq;
      end if;
    end if;
  end process;

  -- Register fip_status

  -- Register presence

  -- Register leds
  leds_val_o <= leds_val_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        leds_val_reg <= "000000";
        leds_wack <= '0';
      else
        if leds_wreq = '1' then
          leds_val_reg <= wr_dat_d0(5 downto 0);
        end if;
        leds_wack <= leds_wreq;
      end if;
    end if;
  end process;

  -- Register boards_pins_0

  -- Register boards_pins_1

  -- Register boards_pins_2

  -- Register boards_pins_3

  -- Register boards_pins_4

  -- Register boards_pins_5

  -- Register boards_pins_6

  -- Register boards_pins_7

  -- Interface fip_reg
  fip_reg_tr <= fip_reg_wt or fip_reg_rt;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        fip_reg_rt <= '0';
        fip_reg_wt <= '0';
      else
        fip_reg_rt <= (fip_reg_rt or fip_reg_re) and not fip_reg_rack;
        fip_reg_wt <= (fip_reg_wt or fip_reg_we) and not fip_reg_wack;
      end if;
    end if;
  end process;
  fip_reg_o.cyc <= fip_reg_tr;
  fip_reg_o.stb <= fip_reg_tr;
  fip_reg_wack <= fip_reg_i.ack and fip_reg_wt;
  fip_reg_rack <= fip_reg_i.ack and fip_reg_rt;
  fip_reg_o.adr <= ((20 downto 0 => '0') & adr_int(10 downto 2)) & (1 downto 0 => '0');
  fip_reg_o.sel <= wr_sel_d0;
  fip_reg_o.we <= fip_reg_wt;
  fip_reg_o.dat <= wr_dat_d0;

  -- Interface plc_mem
  plc_mem_tr <= plc_mem_wt or plc_mem_rt;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        plc_mem_rt <= '0';
        plc_mem_wt <= '0';
      else
        plc_mem_rt <= (plc_mem_rt or plc_mem_re) and not plc_mem_rack;
        plc_mem_wt <= (plc_mem_wt or plc_mem_we) and not plc_mem_wack;
      end if;
    end if;
  end process;
  plc_mem_o.cyc <= plc_mem_tr;
  plc_mem_o.stb <= plc_mem_tr;
  plc_mem_wack <= plc_mem_i.ack and plc_mem_wt;
  plc_mem_rack <= plc_mem_i.ack and plc_mem_rt;
  plc_mem_o.adr <= ((18 downto 0 => '0') & adr_int(12 downto 2)) & (1 downto 0 => '0');
  plc_mem_o.sel <= wr_sel_d0;
  plc_mem_o.we <= plc_mem_wt;
  plc_mem_o.dat <= wr_dat_d0;

  -- Process for write requests.
  process (wr_adr_d0, wr_req_d0, plc_ctrl_wack, leds_wack, fip_reg_wack, plc_mem_wack) begin
    plc_ctrl_wreq <= '0';
    leds_wreq <= '0';
    fip_reg_we <= '0';
    plc_mem_we <= '0';
    case wr_adr_d0(13 downto 13) is
    when "0" => 
      case wr_adr_d0(12 downto 11) is
      when "00" => 
        case wr_adr_d0(10 downto 2) is
        when "000000000" => 
          -- Reg plc_ctrl
          plc_ctrl_wreq <= wr_req_d0;
          wr_ack_int <= plc_ctrl_wack;
        when "000000001" => 
          -- Reg fip_status
          wr_ack_int <= wr_req_d0;
        when "000000010" => 
          -- Reg presence
          wr_ack_int <= wr_req_d0;
        when "000000011" => 
          -- Reg leds
          leds_wreq <= wr_req_d0;
          wr_ack_int <= leds_wack;
        when "000001000" => 
          -- Reg boards_pins_0
          wr_ack_int <= wr_req_d0;
        when "000001001" => 
          -- Reg boards_pins_1
          wr_ack_int <= wr_req_d0;
        when "000001010" => 
          -- Reg boards_pins_2
          wr_ack_int <= wr_req_d0;
        when "000001011" => 
          -- Reg boards_pins_3
          wr_ack_int <= wr_req_d0;
        when "000001100" => 
          -- Reg boards_pins_4
          wr_ack_int <= wr_req_d0;
        when "000001101" => 
          -- Reg boards_pins_5
          wr_ack_int <= wr_req_d0;
        when "000001110" => 
          -- Reg boards_pins_6
          wr_ack_int <= wr_req_d0;
        when "000001111" => 
          -- Reg boards_pins_7
          wr_ack_int <= wr_req_d0;
        when others =>
          wr_ack_int <= wr_req_d0;
        end case;
      when "01" => 
        -- Submap fip_reg
        fip_reg_we <= wr_req_d0;
        wr_ack_int <= fip_reg_wack;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "1" => 
      -- Submap plc_mem
      plc_mem_we <= wr_req_d0;
      wr_ack_int <= plc_mem_wack;
    when others =>
      wr_ack_int <= wr_req_d0;
    end case;
  end process;

  -- Process for read requests.
  process (adr_int, rd_req_int, plc_ctrl_rst_reg, fip_status_var1_rdy_i, fip_status_var3_rdy_i, presence_en_i, leds_val_reg, boards_pins_0_i, boards_pins_1_i, boards_pins_2_i, boards_pins_3_i, boards_pins_4_i, boards_pins_5_i, boards_pins_6_i, boards_pins_7_i, fip_reg_i.dat, fip_reg_rack, plc_mem_i.dat, plc_mem_rack) begin
    -- By default ack read requests
    rd_dat_d0 <= (others => 'X');
    fip_reg_re <= '0';
    plc_mem_re <= '0';
    case adr_int(13 downto 13) is
    when "0" => 
      case adr_int(12 downto 11) is
      when "00" => 
        case adr_int(10 downto 2) is
        when "000000000" => 
          -- Reg plc_ctrl
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0(0) <= plc_ctrl_rst_reg;
          rd_dat_d0(31 downto 1) <= (others => '0');
        when "000000001" => 
          -- Reg fip_status
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0(0) <= fip_status_var1_rdy_i;
          rd_dat_d0(1) <= fip_status_var3_rdy_i;
          rd_dat_d0(31 downto 2) <= (others => '0');
        when "000000010" => 
          -- Reg presence
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0(7 downto 0) <= presence_en_i;
          rd_dat_d0(31 downto 8) <= (others => '0');
        when "000000011" => 
          -- Reg leds
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0(5 downto 0) <= leds_val_reg;
          rd_dat_d0(31 downto 6) <= (others => '0');
        when "000001000" => 
          -- Reg boards_pins_0
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_0_i;
        when "000001001" => 
          -- Reg boards_pins_1
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_1_i;
        when "000001010" => 
          -- Reg boards_pins_2
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_2_i;
        when "000001011" => 
          -- Reg boards_pins_3
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_3_i;
        when "000001100" => 
          -- Reg boards_pins_4
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_4_i;
        when "000001101" => 
          -- Reg boards_pins_5
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_5_i;
        when "000001110" => 
          -- Reg boards_pins_6
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_6_i;
        when "000001111" => 
          -- Reg boards_pins_7
          rd_ack_d0 <= rd_req_int;
          rd_dat_d0 <= boards_pins_7_i;
        when others =>
          rd_ack_d0 <= rd_req_int;
        end case;
      when "01" => 
        -- Submap fip_reg
        fip_reg_re <= rd_req_int;
        rd_dat_d0 <= fip_reg_i.dat;
        rd_ack_d0 <= fip_reg_rack;
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "1" => 
      -- Submap plc_mem
      plc_mem_re <= rd_req_int;
      rd_dat_d0 <= plc_mem_i.dat;
      rd_ack_d0 <= plc_mem_rack;
    when others =>
      rd_ack_d0 <= rd_req_int;
    end case;
  end process;
end syn;
