#ifndef __CHEBY__FIP_URV_REGS__H__
#define __CHEBY__FIP_URV_REGS__H__
#define FIP_URV_REGS_SIZE 16384

/* PLC processor control */
#define FIP_URV_REGS_PLC_CTRL 0x0UL
#define FIP_URV_REGS_PLC_CTRL_RST 0x1UL

/* Status bits from NanoFIP */
#define FIP_URV_REGS_FIP_STATUS 0x4UL
#define FIP_URV_REGS_FIP_STATUS_VAR1_RDY 0x1UL
#define FIP_URV_REGS_FIP_STATUS_VAR3_RDY 0x2UL

/* presence lines for boards */
#define FIP_URV_REGS_PRESENCE 0x8UL
#define FIP_URV_REGS_PRESENCE_EN_MASK 0xffUL
#define FIP_URV_REGS_PRESENCE_EN_SHIFT 0

/* led */
#define FIP_URV_REGS_LEDS 0xcUL
#define FIP_URV_REGS_LEDS_VAL_MASK 0x3fUL
#define FIP_URV_REGS_LEDS_VAL_SHIFT 0

/* None */
#define FIP_URV_REGS_BOARDS 0x20UL
#define FIP_URV_REGS_BOARDS_SIZE 32

/* None */
#define FIP_URV_REGS_BOARDS_PINS_0 0x20UL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_1 0x24UL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_2 0x28UL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_3 0x2cUL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_4 0x30UL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_5 0x34UL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_6 0x38UL

/* None */
#define FIP_URV_REGS_BOARDS_PINS_7 0x3cUL

/* NanoFIP internal memory/registers */
#define FIP_URV_REGS_FIP_REG 0x800UL
#define FIP_URV_REGS_FIP_REG_SIZE 2048

/* Memory of the PLC urv */
#define FIP_URV_REGS_PLC_MEM 0x2000UL
#define FIP_URV_REGS_PLC_MEM_SIZE 8192

struct fip_urv_regs {
  /* [0x0]: REG (rw) PLC processor control */
  uint32_t plc_ctrl;

  /* [0x4]: REG (ro) Status bits from NanoFIP */
  uint32_t fip_status;

  /* [0x8]: REG (ro) presence lines for boards */
  uint32_t presence;

  /* [0xc]: REG (rw) led */
  uint32_t leds;

  /* padding to: 8 words */
  uint32_t __padding_0[4];

  /* [0x20]: BLOCK (no description) */
  struct boards {
    /* [0x0]: REG (ro) (no description) */
    uint32_t pins_0;

    /* [0x4]: REG (ro) (no description) */
    uint32_t pins_1;

    /* [0x8]: REG (ro) (no description) */
    uint32_t pins_2;

    /* [0xc]: REG (ro) (no description) */
    uint32_t pins_3;

    /* [0x10]: REG (ro) (no description) */
    uint32_t pins_4;

    /* [0x14]: REG (ro) (no description) */
    uint32_t pins_5;

    /* [0x18]: REG (ro) (no description) */
    uint32_t pins_6;

    /* [0x1c]: REG (ro) (no description) */
    uint32_t pins_7;
  } boards;

  /* padding to: 512 words */
  uint32_t __padding_1[496];

  /* [0x800]: SUBMAP NanoFIP internal memory/registers */
  uint32_t fip_reg[512];

  /* padding to: 2048 words */
  uint32_t __padding_2[1024];

  /* [0x2000]: SUBMAP Memory of the PLC urv */
  uint32_t plc_mem[2048];
};

#endif /* __CHEBY__FIP_URV_REGS__H__ */
